package fr.cnam.provider;

import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import java.util.*;

import fr.cnam.provider.*;

// Class principal du Provider
public class ProviderImpl extends UnicastRemoteObject implements Provider
{
  private ProxyConsumer[] canaux;  // lLes canaux du provider

  private ProviderImpl() throws Exception
  {
    canaux = new ProxyConsumer[10]; // 10 canaux possibles
    for(int i=0;i<10;i++)
      canaux[i]=new ProxyConsumer();
    Naming.rebind("rmi://localhost:9100/PROVIDER",this);
  }

  // Methode statique utilises par les consommateurs et les producteurs
  //  pour se connecter au provider
  static public Provider getProvider() throws Exception
  {
    return (Provider)(Naming.lookup("rmi://localhost:9100/PROVIDER"));
  }

  // Un producteur se declare a un canal du provider
  public Consumer connectSupplier(Supplier supplier,int canal) throws RemoteException
  {
    System.out.println("Connexion d'un producteur sur le canal "+canal);
    canaux[canal].addSupplier(supplier);
    return((Consumer)(UnicastRemoteObject.toStub(canaux[canal])));
  }

  // Un consommateur se declare a un canal du provider
  public Supplier connectConsumer(Consumer consumer,int canal)  throws RemoteException
  {
    System.out.println("Connexion d'un consommateur sur le canal "+canal);
    return (Supplier)(UnicastRemoteObject.toStub(canaux[canal].addProxySupplier(consumer)));
  }

  // Execution du provider
  public static void main(String[] args) throws Exception
  {
    System.out.println("Provider in run ....");
    try{LocateRegistry.createRegistry(9100);} catch(Exception ex){};
    new ProviderImpl();
  }
}