package fr.cnam.provider;

import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import java.util.*;

// 1 canal = 1 instance de ProxyConsumer
//
// Proxy utilise par les producteurs
public class ProxyConsumer extends UnicastRemoteObject implements Consumer
{
  private ArrayList<Supplier> producteurs;     // Les producteurs connectes a un des canaux du Provider
  private ArrayList<ProxySupplier> suppliers;  // Les proxy utilises pour chacun des consommateurs

  public ProxyConsumer()  throws RemoteException
  {
    super(9101);
    producteurs = new ArrayList<Supplier>();
    suppliers = new ArrayList<ProxySupplier>();
  }

  // Ajout d'un nouveau producteur au canal
  public void addSupplier(Supplier supplier)
  {
    producteurs.add(supplier);
  }

  // Ajout d'un consommateur => creation de son proxy
  public Supplier addProxySupplier(Consumer consumer) throws RemoteException
  {
    ProxySupplier p = new ProxySupplier(consumer,this);
    suppliers.add(p);
    System.out.println("Nb proxy supplier: "+suppliers.size());
    return(p);
  }

  // On demande a tous les proxys de pousser a son tour au consommateur
  public void push(Object obj) throws RemoteException
  {
    for(ProxySupplier p:suppliers)
      try{
        // Dans un thread afin d'etre asynchrone
        new Thread(new Runnable(){public void run(){try{p.push(obj);}catch(Exception ex2){}}}).start();
      }catch(Exception ex){}
  }

  // On tire a chaque producteur 
  public Object pull()  throws RemoteException
  {
    ArrayList list=new ArrayList();
    for(Supplier prod:producteurs)
      {
        try{
          Object obj = prod.pull();
          list.add(obj);
        }catch(Exception ex){}
      }
    return list; // Autant d'evenement tires que de producteurs
  }
}